//
// Created by Laura Majer on 08/04/2020.
//

#include <sstream>
#include "AdaptedPermutation.h"
#include "../../ECF/ECF.h"


namespace AdaptedPermutation {


    void AdaptedPermutation::registerParameters(StateP state) {
        registerParameter(state, "size", (voidP) new uint(1), ECF::UINT,
                          "genotype size: number of indices (mandatory)");
        registerParameter(state,"fleetSize",(voidP) new uint(1), ECF::UINT, "number of available vehicles.");
        registerParameter(state,"problemSize",(voidP) new uint(1), ECF::UINT, "number of available vehicles.");

    }


    bool AdaptedPermutation::initialize(StateP state) {



        voidP genp = getParameterValue(state, "problemSize");
        problemsize_ = *((uint*) genp.get());
        voidP fleetsizep = getParameterValue(state,"fleetSize");
        fleetsize_ = *((uint * ) fleetsizep.get());

        size_ = problemsize_ + fleetsize_ - 1;

        if (size_ < 1) {
            ECF_LOG_ERROR(state, "Error: 'size' must be > 0 for AdaptedPermutation genotype!");
            throw ("");
        }
        variables.resize(static_cast<int>(problemsize_ - 1));

        uint estimate = problemsize_/fleetsize_;

        for(int i=0; i<problemsize_-1; i++) {
            variables[i] = i+1;
        }

        int ind1, ind2, temp;

        // generiranje permutacije od 1 do vel+br.vozila-1
        for (uint i = 0; i < size_; i++) {
            ind1 = state->getRandomizer()->getRandomInteger(problemsize_ - 1);
            ind2 = state->getRandomizer()->getRandomInteger(problemsize_ - 1);
            temp = variables[ind1];
            variables[ind1] = variables[ind2];
            variables[ind2] = temp;
        }

        int vehicle = problemsize_;
        int inserted = 0;
        int offset = 0;

        std::vector<int>::iterator it;

        while(inserted < fleetsize_) {
            it = variables.begin();
            variables.insert((it+offset),vehicle++);
            offset += (estimate + 1);
            inserted++;
        }

        return true;


      /*  for (uint i = 0; i < size_; i++) {
            variables[i] = i+1;
        }
        int ind1, ind2, temp;

        // generiranje permutacije od 1 do vel+br.vozila-1
        for (uint i = 0; i < size_; i++) {
            ind1 = state->getRandomizer()->getRandomInteger(size_);
            ind2 = state->getRandomizer()->getRandomInteger(size_);
            temp = variables[ind1];
            variables[ind1] = variables[ind2];
            variables[ind2] = temp;
        }

        return true;*/
    }


    void AdaptedPermutation::write(XMLNode &xPermutation) {
        xPermutation = XMLNode::createXMLTopNode("AdaptedPermutation");
        std::stringstream sValue;
        sValue << size_;
        xPermutation.addAttribute("size", sValue.str().c_str());

        std::vector<int> perm = variables;

        int i=0;
        //broji se koliko je lokacija navedeno prije prvog vozila
        while(variables[i]<problemsize_) {
            i++;
        }

        int offset = 0;
        while(offset!=i) {
            perm.push_back(variables[offset++]);
        }

        perm.erase(perm.begin(),perm.begin()+i);

        int vind = 1;
        sValue.str("");
        for (uint i = 0; i < size_; i++) {

            //naisli smo na auto
            if(perm[i] >= problemsize_) {
                sValue << "\n Vozilo " << vind++ << ". : ";
            }
            else {
                sValue << "\t" << perm[i];
            }
        }
        xPermutation.addText(sValue.str().c_str());
    }


    void AdaptedPermutation::read(XMLNode &xPermutation) {
        XMLCSTR xIndices = xPermutation.getText();
        std::stringstream sValues;
        sValues << xIndices;

        for (uint index = 0; index < size_; index++) {
            sValues >> variables[index];
        }
    }

}