//
// Created by Laura Majer on 08/04/2020.
//

#ifndef VRP_ADAPTEDPERMUTATION_H
#define VRP_ADAPTEDPERMUTATION_H


#include "../../ECF/ECF.h"
#include "../../ECF/Genotype.h"
#include "AdaptedPermutation.h"
#include "./crossovers/AdaptedPermutationCrsCyclic2.h"
#include "./crossovers/AdaptedPermutationCrsCyclic.h"
#include "./crossovers/AdaptedPermutationCrsDPX.h"
#include "./crossovers/AdaptedPermutationCrsPMX.h"
#include "./crossovers/AdaptedPermutationCrsSPX.h"
#include "./crossovers/AdaptedPermutationCrsOBX.h"
#include "./crossovers/AdaptedPermutationCrsOPX.h"
#include "./crossovers/AdaptedPermutationCrsOX.h"
#include "./crossovers/AdaptedPermutationCrsOX2.h"
#include "./mutators/AdaptedPermutationMutIns.h"
#include "./mutators/AdaptedPermutationMutToggle.h"



namespace AdaptedPermutation {

class AdaptedPermutation: public Genotype {

    protected:
        uint size_;	//!< permutation vector size
        uint fleetsize_; //number of available vehicles
        uint problemsize_;
    public:
        typedef std::vector<int> var;
        var variables;

        AdaptedPermutation () {name_ = "AdaptedPermutation";}

        uint getSize()	//!< return size of indices vector
        {	return size_;	}

        uint getFleetSize()	//!< return size of indices vector
        {	return fleetsize_;	}

        uint getProblemSize()	//!< return size of indices vector
        {	return problemsize_;	}




        bool initialize (StateP state);

        void registerParameters(StateP state);

        AdaptedPermutation* copy()
        {
            AdaptedPermutation *newObject = new AdaptedPermutation(*this);
            return newObject;
        }

        std::vector<CrossoverOpP> getCrossoverOp()
        {
            std::vector<CrossoverOpP> crx;
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsCyclic));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsCyclic2));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsPMX));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsOPX));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsSPX));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsDPX));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsOX));
            crx.push_back(static_cast<CrossoverOpP> (new AdaptedPermutationCrsOX2));
            return crx;
        }

        std::vector<MutationOpP> getMutationOp()
        {
            std::vector<MutationOpP> mut;
            mut.push_back(static_cast<MutationOpP> (new AdaptedPermutationMutToggle));
            mut.push_back(static_cast<MutationOpP> (new AdaptedPermutationMutIns));
            return mut;

        }

        void write(XMLNode&);
        void read(XMLNode&);
    };
}

typedef boost::shared_ptr<AdaptedPermutation::AdaptedPermutation> AdaptedPermutationP;

#endif //VRP_ADAPTEDPERMUTATION_H
