#include "ECF/ECF_base.h"
#include "NewPermutation.h"
#include<sstream>
#include <ECF/Genotype.h>


namespace NewPermutation
{

void NewPermutation::registerParameters(StateP state)
{
	registerParameter(state, "size", (voidP) new uint(1), ECF::UINT,
		"genotype size: number of indices (mandatory)");
	registerParameter(state, "schedule", (voidP) new std::string, ECF::STRING,
		"final order of locations per vehicle");
    registerParameter(state, "double.check", (voidP) new uint(1), ECF::UINT,
                      "is the genotype evaluated once or twice (backwards)");

}


bool NewPermutation::initialize (StateP state)
{
	voidP genp = getParameterValue(state, "size");
	size_ = *((uint*) genp.get());
    voidP doublep = getParameterValue(state, "double.check");
    double_check = *((uint*) doublep.get());


	if(size_ < 1) {
		ECF_LOG_ERROR(state, "Error: 'size' must be > 0 for Permutation genotype!");
		throw("");
	}
	variables.resize(static_cast<int>(size_));

	for(uint i = 0; i < size_; i++) {
		variables[i] = i+1;
	}
	int ind1, ind2, temp;

	// generate random permutation
	for(uint i = 0; i < size_; i++) {
		ind1 = state->getRandomizer()->getRandomInteger(size_);
		ind2 = state->getRandomizer()->getRandomInteger(size_);
		temp = variables[ind1];
		variables[ind1] = variables[ind2];
		variables[ind2] = temp;
	}
	return true;
}


void NewPermutation::write(XMLNode &xPermutation)
{
	xPermutation = XMLNode::createXMLTopNode("NewPermutation");

	xPermutation.addText(schedule.c_str());
}


void NewPermutation::read(XMLNode &xPermutation)
{
	XMLCSTR xIndices = xPermutation.getText();
	std::stringstream sValues;
	sValues << xIndices;

	for(uint index = 0; index < size_; index++) {
		sValues >> variables[index];
	}
}

}