#ifndef NEWPERMUTATION_H_
#define NEWPERMUTATION_H_

#include "./crossovers/NewPermutationCrsCyclic.h"
#include "./crossovers/NewPermutationCrsDPX.h"
#include "./crossovers/NewPermutationCrsCyclic2.h"
#include "./crossovers/NewPermutationCrsOBX.h"
#include "./crossovers/NewPermutationCrsOPX.h"
#include "./crossovers/NewPermutationCrsOX.h"
#include "./crossovers/NewPermutationCrsOX2.h"
#include "./crossovers/NewPermutationCrsPBX.h"
#include "./crossovers/NewPermutationCrsPMX.h"
#include "./crossovers/NewPermutationCrsSPX.h"
#include "./crossovers/NewPermutationCrsUPMX.h"
#include "./mutators/NewPermutationMutIns.h"
#include "./mutators/NewPermutationMutInv.h"
#include "./mutators/NewPermutationMutToggle.h"



#include "ECF/Genotype.h"


namespace NewPermutation
{

/**
 * \defgroup permutation Permutation
 * \ingroup genotypes
 */

/**
 * \ingroup genotypes permutation
 * \brief Permutation class - implements genotype as a vector of indices (permutation of indices)
 */
class NewPermutation: public Genotype {
protected:
	uint size_;	//!< permutation vector size
	std::string schedule;
	uint double_check;
public:
	typedef std::vector<int> var;
	var variables;

	NewPermutation () {name_ = "NewPermutation";    }

	uint getSize()	//!< return size of indices vector
	{	return size_;	}

	uint getDoubleCheck() {
	    return double_check;
	}

	void setSchedule(std::string sched)
    {   schedule.resize(sched.size());
        schedule = sched;   }

	bool initialize (StateP state);

	void registerParameters(StateP state);

	NewPermutation* copy()
	{
		NewPermutation *newObject = new NewPermutation(*this);
		return newObject;
	}

	std::vector<CrossoverOpP> getCrossoverOp()
	{
		std::vector<CrossoverOpP> crx;
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsCyclic));
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsDPX));
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsCyclic2));
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsOX));
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsOX2));
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsPMX));
		crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsSPX));
		//crx.push_back(static_cast<CrossoverOpP> (new NewPermutationCrsUPMX));
		return crx;
	}

	std::vector<MutationOpP> getMutationOp()
	{
		std::vector<MutationOpP> mut;
		mut.push_back(static_cast<MutationOpP> (new NewPermutationMutIns));
		mut.push_back(static_cast<MutationOpP> (new NewPermutationMutInv));
		mut.push_back(static_cast<MutationOpP> (new NewPermutationMutToggle));
		return mut;

	}

	void write(XMLNode&);
	void read(XMLNode&);
};
}

typedef boost::shared_ptr<NewPermutation::NewPermutation> NewPermutationP;
#endif
